NO_COMMAND = -1
STITCH = 0
JUMP = 1
TRIM = 2
STOP = 3
END = 4
COLOR_CHANGE = 5
SEQUIN_MODE = 6
SEQUIN_EJECT = 7

# Stitch with implied contingency.
SEW_TO = 0xB0
NEEDLE_AT = 0xB1

SEQUENCE_BREAK = 0xE1
COLOR_BREAK = 0xE2
TIE_ON = 0xE4
TIE_OFF = 0xE5
FRAME_EJECT = 0xE9

MATRIX_TRANSLATE = 0xC0
MATRIX_SCALE = 0xC1
MATRIX_ROTATE = 0xC2
MATRIX_RESET = 0xC3

OPTION_ENABLE_TIE_ON = 0xD1
OPTION_ENABLE_TIE_OFF = 0xD2
OPTION_DISABLE_TIE_ON = 0xD3
OPTION_DISABLE_TIE_OFF = 0xD4
OPTION_MAX_STITCH_LENGTH = 0xD5
OPTION_MAX_JUMP_LENGTH = 0xD6
OPTION_EXPLICIT_TRIM = 0xD7
OPTION_IMPLICIT_TRIM = 0xD8

CONTINGENCY_NONE = 0xF0
CONTINGENCY_JUMP_NEEDLE = 0xF1
CONTINGENCY_SEW_TO = 0xF2

# Eventually the commands are supposed to be limited to 255 thereby
# allowing additional information like, color change to color in position n
# To be stored in the higher level bits.
COMMAND_MASK = 0xFF
